@@ -26,7 +26,7 @@ namespace pqxx
 [[nodiscard]] std::string::size_type
 array_parser::scan_glyph(std::string::size_type pos) const
 {
-  return m_scan(m_input.data(), std::size(m_input), pos);
+  return m_scan(std::data(m_input), std::size(m_input), pos);
 }
 
 
@@ -34,7 +34,7 @@ array_parser::scan_glyph(std::string::size_type pos) const
 std::string::size_type array_parser::scan_glyph(
   std::string::size_type pos, std::string::size_type end) const
 {
-  return m_scan(m_input.data(), end, pos);
+  return m_scan(std::data(m_input), end, pos);
 }
 
 
@@ -50,33 +50,34 @@ std::string::size_type array_parser::scan_single_quoted_string() const
        here = next, next = scan_glyph(here))
   {
     if (next - here == 1)
-      switch (m_input[here])
+      PQXX_LIKELY
+    switch (m_input[here])
+    {
+    case '\'':
+      // SQL escapes single quotes by doubling them.  Terrible idea, but it's
+      // what we have.  Inspect the next character to find out whether this
+      // is the closing quote, or an escaped one inside the string.
+      here = next;
+      // (We can read beyond this quote because the array will always end in
+      // a closing brace.)
+      next = scan_glyph(here);
+
+      if ((here + 1 < next) or (m_input[here] != '\''))
       {
-      case '\'':
-        // SQL escapes single quotes by doubling them.  Terrible idea, but it's
-        // what we have.  Inspect the next character to find out whether this
-        // is the closing quote, or an escaped one inside the string.
-        here = next;
-        // (We can read beyond this quote because the array will always end in
-        // a closing brace.)
-        next = scan_glyph(here);
-
-        if ((here + 1 < next) or (m_input[here] != '\''))
-        {
-          // Our lookahead character is not an escaped quote.  It's the first
-          // character outside our string.  So, return it.
-          return here;
-        }
-
-        // We've just scanned an escaped quote.  Keep going.
-        break;
-
-      case '\\':
-        // Backslash escape.  Skip ahead by one more character.
-        here = next;
-        next = scan_glyph(here);
-        break;
+        // Our lookahead character is not an escaped quote.  It's the first
+        // character outside our string.  So, return it.
+        return here;
       }
+
+      // We've just scanned an escaped quote.  Keep going.
+      break;
+
+    case '\\':
+      // Backslash escape.  Skip ahead by one more character.
+      here = next;
+      next = scan_glyph(here);
+      break;
+    }
   }
   throw argument_error{internal::concat("Null byte in SQL string: ", m_input)};
 }
@@ -96,12 +97,14 @@ array_parser::parse_single_quoted_string(std::string::size_type end) const
   {
     if (next - here == 1 and (m_input[here] == '\'' or m_input[here] == '\\'))
     {
-      // Skip escape.
+      // Skip escape.  (Performance-wise, we bet that these are relatively
+      // rare.)
+      PQXX_UNLIKELY
       here = next;
       next = scan_glyph(here, end);
     }
 
-    output.append(m_input.data() + here, m_input.data() + next);
+    output.append(std::data(m_input) + here, std::data(m_input) + next);
   }
 
   return output;
@@ -112,7 +115,7 @@ array_parser::parse_single_quoted_string(std::string::size_type end) const
 std::string::size_type array_parser::scan_double_quoted_string() const
 {
   return pqxx::internal::scan_double_quoted_string(
-    m_input.data(), std::size(m_input), m_pos, m_scan);
+    std::data(m_input), std::size(m_input), m_pos, m_scan);
 }
 
 
@@ -121,7 +124,7 @@ std::string
 array_parser::parse_double_quoted_string(std::string::size_type end) const
 {
   return pqxx::internal::parse_double_quoted_string(
-    m_input.data(), end, m_pos, m_scan);
+    std::data(m_input), end, m_pos, m_scan);
 }
 
 
@@ -131,7 +134,7 @@ array_parser::parse_double_quoted_string(std::string::size_type end) const
 std::string::size_type array_parser::scan_unquoted_string() const
 {
   return pqxx::internal::scan_unquoted_string<',', ';', '}'>(
-    m_input.data(), std::size(m_input), m_pos, m_scan);
+    std::data(m_input), std::size(m_input), m_pos, m_scan);
 }
 
 
@@ -143,7 +146,7 @@ std::string
 array_parser::parse_unquoted_string(std::string::size_type end) const
 {
   return pqxx::internal::parse_unquoted_string(
-    m_input.data(), end, m_pos, m_scan);
+    std::data(m_input), end, m_pos, m_scan);
 }
 
 
@@ -206,6 +209,7 @@ std::pair<array_parser::juncture, std::string> array_parser::get_next()
       {
         // The normal case: we just parsed an unquoted string.  The value is
         // what we need.
+        PQXX_LIKELY
         found = juncture::string_value;
       }
       break;
@@ -216,7 +220,8 @@ std::pair<array_parser::juncture, std::string> array_parser::get_next()
   {
     auto next{scan_glyph(end)};
     if (next - end == 1 and (m_input[end] == ',' or m_input[end] == ';'))
-      end = next;
+      PQXX_UNLIKELY
+    end = next;
   }
 
   m_pos = end;
