@@ -116,8 +116,9 @@ pqxx::blob::~blob()
   catch (std::exception const &e)
   {
     if (m_conn != nullptr)
-      m_conn->process_notice(internal::concat(
-        "Failure while closing binary large object: ", e.what(), "\n"));
+      PQXX_UNLIKELY
+    m_conn->process_notice(internal::concat(
+      "Failure while closing binary large object: ", e.what(), "\n"));
   }
 }
 
@@ -153,21 +154,21 @@ std::size_t
 pqxx::blob::read(std::basic_string<std::byte> &buf, std::size_t size)
 {
   buf.resize(size);
-  auto const received{raw_read(buf.data(), size)};
+  auto const received{raw_read(std::data(buf), size)};
   buf.resize(received);
   return static_cast<std::size_t>(received);
 }
 
 
-void pqxx::blob::write(std::basic_string_view<std::byte> buf)
+void pqxx::blob::raw_write(std::byte const buf[], std::size_t size)
 {
   if (m_conn == nullptr)
     throw usage_error{"Attempt to write to a closed binary large object."};
-  if (std::size(buf) > chunk_limit)
+  if (size > chunk_limit)
     throw range_error{
       "Writes to a binary large object must be less than 2 GB at once."};
-  auto ptr{reinterpret_cast<char const *>(buf.data())};
-  int written{lo_write(raw_conn(m_conn), m_fd, ptr, std::size(buf))};
+  auto ptr{reinterpret_cast<char const *>(buf)};
+  int written{lo_write(raw_conn(m_conn), m_fd, ptr, size)};
   if (written < 0)
     throw failure{
       internal::concat("Write to binary large object failed: ", errmsg())};
@@ -290,7 +291,7 @@ std::size_t pqxx::blob::append_to_buf(
   buf.resize(org_size + append_max);
   try
   {
-    auto here{reinterpret_cast<char *>(buf.data() + org_size)};
+    auto here{reinterpret_cast<char *>(std::data(buf) + org_size)};
     auto chunk{static_cast<std::size_t>(
       lo_read(b.raw_conn(b.m_conn), b.m_fd, here, append_max))};
     buf.resize(org_size + chunk);
