@@ -275,6 +275,7 @@ void pqxx::connection::set_up_state()
   // don't do that by default because there's a danger: libpq may call the
   // notice processor via a result object, even after the connection has been
   // destroyed and the handlers list no longer exists.
+  PQXX_LIKELY
   PQsetNoticeProcessor(m_conn, inert_notice_processor, nullptr);
 }
 
@@ -307,7 +308,8 @@ void pqxx::connection::process_notice(char const msg[]) noexcept
     process_notice_raw(msg);
   else
     // Newline is missing.  Let the zview version of the code add it.
-    process_notice(view);
+    PQXX_UNLIKELY
+  process_notice(view);
 }
 
 
@@ -330,6 +332,7 @@ void pqxx::connection::process_notice(zview msg) noexcept
     catch (std::exception const &)
     {
       // If nothing else works, try writing the message without the newline.
+      PQXX_UNLIKELY
       process_notice_raw(msg.c_str());
     }
 }
@@ -385,6 +388,7 @@ void pqxx::connection::remove_receiver(pqxx::notification_receiver *T) noexcept
 
     if (i == R.second)
     {
+      PQXX_UNLIKELY
       process_notice(internal::concat(
         "Attempt to remove unknown receiver '", needle.first, "'"));
     }
@@ -400,6 +404,7 @@ void pqxx::connection::remove_receiver(pqxx::notification_receiver *T) noexcept
   }
   catch (std::exception const &e)
   {
+    PQXX_UNLIKELY
     process_notice(e.what());
   }
 }
@@ -422,14 +427,16 @@ void pqxx::connection::cancel_query()
   using pointer = std::unique_ptr<PGcancel, std::function<void(PGcancel *)>>;
   pointer cancel{PQgetCancel(m_conn), PQfreeCancel};
   if (cancel == nullptr)
-    throw std::bad_alloc{};
+    PQXX_UNLIKELY
+  throw std::bad_alloc{};
 
   std::array<char, 500u> errbuf;
-  auto const c{PQcancel(
-    cancel.get(), errbuf.data(), static_cast<int>(std::size(errbuf)))};
+  auto const err{errbuf.data()};
+  auto const c{
+    PQcancel(cancel.get(), err, static_cast<int>(std::size(errbuf)))};
   if (c == 0)
-    throw pqxx::sql_error{
-      std::string{errbuf.data(), std::size(errbuf)}, "[cancel]"};
+    PQXX_UNLIKELY
+  throw pqxx::sql_error{std::string{err, std::size(errbuf)}, "[cancel]"};
 }
 
 
@@ -456,12 +463,14 @@ notify_ptr get_notif(pqxx::internal::pq::PGconn *conn)
 int pqxx::connection::get_notifs()
 {
   if (not consume_input())
-    throw broken_connection{"Connection lost."};
+    PQXX_UNLIKELY
+  throw broken_connection{"Connection lost."};
 
   // Even if somehow we receive notifications during our transaction, don't
   // deliver them.
   if (m_trans)
-    return 0;
+    PQXX_UNLIKELY
+  return 0;
 
   int notifs = 0;
   for (auto N{get_notif(m_conn)}; N.get(); N = get_notif(m_conn))
@@ -657,13 +666,15 @@ void pqxx::connection::close()
   try
   {
     if (m_trans)
-      process_notice(internal::concat(
-        "Closing connection while ",
-        internal::describe_object("transaction"sv, m_trans->name()),
-        " is still open."));
+      PQXX_UNLIKELY
+    process_notice(internal::concat(
+      "Closing connection while ",
+      internal::describe_object("transaction"sv, m_trans->name()),
+      " is still open."));
 
     if (not std::empty(m_receivers))
     {
+      PQXX_UNLIKELY
       process_notice("Closing connection with outstanding receivers.");
       m_receivers.clear();
     }
@@ -766,9 +777,11 @@ void pqxx::connection::write_copy_line(std::string_view line)
   auto const size{check_cast<int>(
     internal::ssize(line), "Line in stream_to is too long to process."sv)};
   if (PQputCopyData(m_conn, line.data(), size) <= 0)
-    throw failure{err_prefix + err_msg()};
+    PQXX_UNLIKELY
+  throw failure{err_prefix + err_msg()};
   if (PQputCopyData(m_conn, "\n", 1) <= 0)
-    throw failure{err_prefix + err_msg()};
+    PQXX_UNLIKELY
+  throw failure{err_prefix + err_msg()};
 }
 
 
@@ -797,7 +810,8 @@ void pqxx::connection::end_copy_write()
 void pqxx::connection::start_exec(char const query[])
 {
   if (PQsendQuery(m_conn, query) == 0)
-    throw failure{err_msg()};
+    PQXX_UNLIKELY
+  throw failure{err_msg()};
 }
 
 
@@ -813,7 +827,8 @@ size_t pqxx::connection::esc_to_buf(std::string_view text, char *buf) const
   auto const copied{
     PQescapeStringConn(m_conn, buf, text.data(), std::size(text), &err)};
   if (err)
-    throw argument_error{err_msg()};
+    PQXX_UNLIKELY
+  throw argument_error{err_msg()};
   return copied;
 }
 
@@ -831,16 +846,14 @@ std::string pqxx::connection::esc(std::string_view text) const
 std::string
 pqxx::connection::esc_raw(unsigned char const bin[], std::size_t len) const
 {
-  return pqxx::internal::esc_bin(
-    std::string_view{reinterpret_cast<char const *>(bin), len});
+  return pqxx::internal::esc_bin(binary_cast(bin, len));
 }
 
 
 std::string
 pqxx::connection::esc_raw(std::basic_string_view<std::byte> bin) const
 {
-  return pqxx::internal::esc_bin(std::string_view{
-    reinterpret_cast<char const *>(bin.data()), std::size(bin)});
+  return pqxx::internal::esc_bin(bin);
 }
 
 
@@ -849,7 +862,11 @@ std::string pqxx::connection::unesc_raw(char const text[]) const
   if (text[0] == '\\' and text[1] == 'x')
   {
     // Hex-escaped format.
-    return pqxx::internal::unesc_bin(std::string_view{text});
+    std::string buf;
+    buf.resize(pqxx::internal::size_unesc_bin(std::strlen(text)));
+    pqxx::internal::unesc_bin(
+      std::string_view{text}, reinterpret_cast<std::byte *>(buf.data()));
+    return buf;
   }
   else
   {
@@ -868,9 +885,7 @@ std::string pqxx::connection::unesc_raw(char const text[]) const
 std::string
 pqxx::connection::quote_raw(unsigned char const bin[], std::size_t len) const
 {
-  std::basic_string_view<std::byte> view{
-    reinterpret_cast<std::byte const *>(bin), len};
-  return internal::concat("'", esc_raw(view), "'::bytea");
+  return internal::concat("'", esc_raw(binary_cast(bin, len)), "'::bytea");
 }
 
 
@@ -899,7 +914,8 @@ std::string pqxx::connection::quote_name(std::string_view identifier) const
     PQescapeIdentifier(m_conn, identifier.data(), std::size(identifier)),
     PQfreemem};
   if (buf.get() == nullptr)
-    throw failure{err_msg()};
+    PQXX_UNLIKELY
+  throw failure{err_msg()};
   return std::string{buf.get()};
 }
 
@@ -927,7 +943,9 @@ pqxx::connection::esc_like(std::string_view text, char escape_char) const
     internal::enc_group(encoding_id()),
     [&out, escape_char](char const *gbegin, char const *gend) {
       if ((gend - gbegin == 1) and (*gbegin == '_' or *gbegin == '%'))
-        out.push_back(escape_char);
+        // We're not expecting a lot of wildcards in a string.  Usually.
+        PQXX_UNLIKELY
+      out.push_back(escape_char);
 
       for (; gbegin != gend; ++gbegin) out.push_back(*gbegin);
     },
@@ -953,7 +971,8 @@ namespace
 void wait_fd(int fd, bool forwrite = false, timeval *tv = nullptr)
 {
   if (fd < 0)
-    throw pqxx::broken_connection{"No connection."};
+    PQXX_UNLIKELY
+  throw pqxx::broken_connection{"No connection."};
 
 // WSAPoll is available in winsock2.h only for versions of Windows >= 0x0600
 #if defined(_WIN32) && (_WIN32_WINNT >= 0x0600)
@@ -1031,6 +1050,7 @@ int pqxx::connection::await_notification()
   int notifs = get_notifs();
   if (notifs == 0)
   {
+    PQXX_LIKELY
     wait_read();
     notifs = get_notifs();
   }
@@ -1044,6 +1064,7 @@ int pqxx::connection::await_notification(
   int notifs = get_notifs();
   if (notifs == 0)
   {
+    PQXX_LIKELY
     wait_read(seconds, microseconds);
     return get_notifs();
   }
@@ -1055,18 +1076,9 @@ std::string pqxx::connection::adorn_name(std::string_view n)
 {
   auto const id{to_string(++m_unique_id)};
   if (std::empty(n))
-  {
-    return "x" + id;
-  }
+    return pqxx::internal::concat("x", id);
   else
-  {
-    std::string name;
-    name.reserve(std::size(n) + 1 + std::size(id));
-    name.append(n);
-    name.push_back('_');
-    name.append(id);
-    return name;
-  }
+    return pqxx::internal::concat(n, "_", id);
 }
 
 
@@ -1082,13 +1094,16 @@ void pqxx::connection::set_client_encoding(char const encoding[])
   {
   case 0:
     // OK.
+    PQXX_LIKELY
     break;
   case -1:
+    PQXX_UNLIKELY
     if (is_open())
       throw failure{"Setting client encoding failed."};
     else
       throw broken_connection{"Lost connection to the database server."};
   default:
+    PQXX_UNLIKELY
     throw internal_error{internal::concat(
       "Unexpected result from PQsetClientEncoding: ", retval)};
   }
@@ -1105,11 +1120,13 @@ int pqxx::connection::encoding_id() const
     // *before* checking a query result for failure.  So, we need to handle
     // connection failure here and it will apply in lots of places.
     // TODO: Make pqxx::result::result(...) do all the checking.
+    PQXX_UNLIKELY
     if (is_open())
       throw failure{"Could not obtain client encoding."};
     else
       throw broken_connection{"Lost connection to the database server."};
   }
+  PQXX_LIKELY
   return enc;
 }
 
@@ -1159,13 +1176,15 @@ char const *get_default(PQconninfoOption const &opt) noexcept
 std::string pqxx::connection::connection_string() const
 {
   if (m_conn == nullptr)
-    throw usage_error{"Can't get connection string: connection is not open."};
+    PQXX_UNLIKELY
+  throw usage_error{"Can't get connection string: connection is not open."};
 
   std::unique_ptr<
     PQconninfoOption, std::function<void(PQconninfoOption *)>> const params{
     PQconninfo(m_conn), PQconninfoFree};
   if (params.get() == nullptr)
-    throw std::bad_alloc{};
+    PQXX_UNLIKELY
+  throw std::bad_alloc{};
 
   std::string buf;
   for (std::size_t i{0}; params.get()[i].keyword != nullptr; ++i)
