@@ -85,13 +85,15 @@ PQXX_PURE std::size_t next_seq_for_euc_jplike(
     return start + 1;
 
   if (start + 2 > buffer_len)
-    throw_for_encoding_error(encoding_name, buffer, start, 1);
+    PQXX_UNLIKELY
+  throw_for_encoding_error(encoding_name, buffer, start, 1);
 
   auto const byte2{get_byte(buffer, start + 1)};
   if (byte1 == 0x8e)
   {
     if (not between_inc(byte2, 0xa1, 0xfe))
-      throw_for_encoding_error(encoding_name, buffer, start, 2);
+      PQXX_UNLIKELY
+    throw_for_encoding_error(encoding_name, buffer, start, 2);
 
     return start + 2;
   }
@@ -99,7 +101,8 @@ PQXX_PURE std::size_t next_seq_for_euc_jplike(
   if (between_inc(byte1, 0xa1, 0xfe))
   {
     if (not between_inc(byte2, 0xa1, 0xfe))
-      throw_for_encoding_error(encoding_name, buffer, start, 2);
+      PQXX_UNLIKELY
+    throw_for_encoding_error(encoding_name, buffer, start, 2);
 
     return start + 2;
   }
@@ -109,7 +112,8 @@ PQXX_PURE std::size_t next_seq_for_euc_jplike(
     auto const byte3{get_byte(buffer, start + 2)};
     if (
       not between_inc(byte2, 0xa1, 0xfe) or not between_inc(byte3, 0xa1, 0xfe))
-      throw_for_encoding_error(encoding_name, buffer, start, 3);
+      PQXX_UNLIKELY
+    throw_for_encoding_error(encoding_name, buffer, start, 3);
 
     return start + 3;
   }
@@ -140,18 +144,22 @@ PQXX_PURE std::size_t next_seq_for_sjislike(
 
   if (
     not between_inc(byte1, 0x81, 0x9f) and not between_inc(byte1, 0xe0, 0xfc))
-    throw_for_encoding_error(encoding_name, buffer, start, 1);
+    PQXX_UNLIKELY
+  throw_for_encoding_error(encoding_name, buffer, start, 1);
 
   if (start + 2 > buffer_len)
-    throw_for_encoding_error(encoding_name, buffer, start, buffer_len - start);
+    PQXX_UNLIKELY
+  throw_for_encoding_error(encoding_name, buffer, start, buffer_len - start);
 
   auto const byte2{get_byte(buffer, start + 1)};
   if (byte2 == 0x7f)
-    throw_for_encoding_error(encoding_name, buffer, start, 2);
+    PQXX_UNLIKELY
+  throw_for_encoding_error(encoding_name, buffer, start, 2);
 
   if (between_inc(byte2, 0x40, 0x9e) or between_inc(byte2, 0x9f, 0xfc))
     return start + 2;
 
+  PQXX_UNLIKELY
   throw_for_encoding_error(encoding_name, buffer, start, 2);
 }
 } // namespace
@@ -191,12 +199,14 @@ PQXX_PURE std::size_t glyph_scanner<encoding_group::BIG5>::call(
     return start + 1;
 
   if (not between_inc(byte1, 0x81, 0xfe) or (start + 2 > buffer_len))
-    throw_for_encoding_error("BIG5", buffer, start, 1);
+    PQXX_UNLIKELY
+  throw_for_encoding_error("BIG5", buffer, start, 1);
 
   auto const byte2{get_byte(buffer, start + 1)};
   if (
     not between_inc(byte2, 0x40, 0x7e) and not between_inc(byte2, 0xa1, 0xfe))
-    throw_for_encoding_error("BIG5", buffer, start, 2);
+    PQXX_UNLIKELY
+  throw_for_encoding_error("BIG5", buffer, start, 2);
 
   return start + 2;
 }
@@ -225,11 +235,13 @@ PQXX_PURE std::size_t glyph_scanner<encoding_group::EUC_CN>::call(
     return start + 1;
 
   if (not between_inc(byte1, 0xa1, 0xf7) or start + 2 > buffer_len)
-    throw_for_encoding_error("EUC_CN", buffer, start, 1);
+    PQXX_UNLIKELY
+  throw_for_encoding_error("EUC_CN", buffer, start, 1);
 
   auto const byte2{get_byte(buffer, start + 1)};
   if (not between_inc(byte2, 0xa1, 0xfe))
-    throw_for_encoding_error("EUC_CN", buffer, start, 2);
+    PQXX_UNLIKELY
+  throw_for_encoding_error("EUC_CN", buffer, start, 2);
 
   return start + 2;
 }
@@ -261,11 +273,13 @@ PQXX_PURE std::size_t glyph_scanner<encoding_group::EUC_KR>::call(
     return start + 1;
 
   if (not between_inc(byte1, 0xa1, 0xfe) or start + 2 > buffer_len)
-    throw_for_encoding_error("EUC_KR", buffer, start, 1);
+    PQXX_UNLIKELY
+  throw_for_encoding_error("EUC_KR", buffer, start, 1);
 
   auto const byte2{get_byte(buffer, start + 1)};
   if (not between_inc(byte2, 0xa1, 0xfe))
-    throw_for_encoding_error("EUC_KR", buffer, start, 1);
+    PQXX_UNLIKELY
+  throw_for_encoding_error("EUC_KR", buffer, start, 1);
 
   return start + 2;
 }
@@ -276,26 +290,30 @@ PQXX_PURE std::size_t glyph_scanner<encoding_group::EUC_TW>::call(
   char const buffer[], std::size_t buffer_len, std::size_t start)
 {
   if (start >= buffer_len)
-    return std::string::npos;
+    PQXX_UNLIKELY
+  return std::string::npos;
 
   auto const byte1{get_byte(buffer, start)};
   if (byte1 < 0x80)
     return start + 1;
 
   if (start + 2 > buffer_len)
-    throw_for_encoding_error("EUC_KR", buffer, start, 1);
+    PQXX_UNLIKELY
+  throw_for_encoding_error("EUC_KR", buffer, start, 1);
 
   auto const byte2{get_byte(buffer, start + 1)};
   if (between_inc(byte1, 0xa1, 0xfe))
   {
     if (not between_inc(byte2, 0xa1, 0xfe))
-      throw_for_encoding_error("EUC_KR", buffer, start, 2);
+      PQXX_UNLIKELY
+    throw_for_encoding_error("EUC_KR", buffer, start, 2);
 
     return start + 2;
   }
 
   if (byte1 != 0x8e or start + 4 > buffer_len)
-    throw_for_encoding_error("EUC_KR", buffer, start, 1);
+    PQXX_UNLIKELY
+  throw_for_encoding_error("EUC_KR", buffer, start, 1);
 
   if (
     between_inc(byte2, 0xa1, 0xb0) and
@@ -303,6 +321,7 @@ PQXX_PURE std::size_t glyph_scanner<encoding_group::EUC_TW>::call(
     between_inc(get_byte(buffer, start + 3), 0xa1, 0xfe))
     return start + 4;
 
+  PQXX_UNLIKELY
   throw_for_encoding_error("EUC_KR", buffer, start, 4);
 }
 
@@ -319,19 +338,22 @@ PQXX_PURE std::size_t glyph_scanner<encoding_group::GB18030>::call(
     return start + 1;
 
   if (start + 2 > buffer_len)
-    throw_for_encoding_error("GB18030", buffer, start, buffer_len - start);
+    PQXX_UNLIKELY
+  throw_for_encoding_error("GB18030", buffer, start, buffer_len - start);
 
   auto const byte2{get_byte(buffer, start + 1)};
   if (between_inc(byte2, 0x40, 0xfe))
   {
     if (byte2 == 0x7f)
-      throw_for_encoding_error("GB18030", buffer, start, 2);
+      PQXX_UNLIKELY
+    throw_for_encoding_error("GB18030", buffer, start, 2);
 
     return start + 2;
   }
 
   if (start + 4 > buffer_len)
-    throw_for_encoding_error("GB18030", buffer, start, buffer_len - start);
+    PQXX_UNLIKELY
+  throw_for_encoding_error("GB18030", buffer, start, buffer_len - start);
 
   if (
     between_inc(byte2, 0x30, 0x39) and
@@ -339,6 +361,7 @@ PQXX_PURE std::size_t glyph_scanner<encoding_group::GB18030>::call(
     between_inc(get_byte(buffer, start + 3), 0x30, 0x39))
     return start + 4;
 
+  PQXX_UNLIKELY
   throw_for_encoding_error("GB18030", buffer, start, 4);
 }
 
@@ -355,7 +378,8 @@ PQXX_PURE std::size_t glyph_scanner<encoding_group::GBK>::call(
     return start + 1;
 
   if (start + 2 > buffer_len)
-    throw_for_encoding_error("GBK", buffer, start, 1);
+    PQXX_UNLIKELY
+  throw_for_encoding_error("GBK", buffer, start, 1);
 
   auto const byte2{get_byte(buffer, start + 1)};
   if (
@@ -373,6 +397,7 @@ PQXX_PURE std::size_t glyph_scanner<encoding_group::GBK>::call(
      byte2 != 0x7f))
     return start + 2;
 
+  PQXX_UNLIKELY
   throw_for_encoding_error("GBK", buffer, start, 2);
 }
 
@@ -397,7 +422,8 @@ PQXX_PURE std::size_t glyph_scanner<encoding_group::JOHAB>::call(
     return start + 1;
 
   if (start + 2 > buffer_len)
-    throw_for_encoding_error("JOHAB", buffer, start, 1);
+    PQXX_UNLIKELY
+  throw_for_encoding_error("JOHAB", buffer, start, 1);
 
   auto const byte2{get_byte(buffer, start)};
   if (
@@ -407,6 +433,7 @@ PQXX_PURE std::size_t glyph_scanner<encoding_group::JOHAB>::call(
      (between_inc(byte2, 0x31, 0x7e) or between_inc(byte2, 0x91, 0xfe))))
     return start + 2;
 
+  PQXX_UNLIKELY
   throw_for_encoding_error("JOHAB", buffer, start, 2);
 }
 
@@ -429,14 +456,16 @@ PQXX_PURE std::size_t glyph_scanner<encoding_group::MULE_INTERNAL>::call(
     return start + 1;
 
   if (start + 2 > buffer_len)
-    throw_for_encoding_error("MULE_INTERNAL", buffer, start, 1);
+    PQXX_UNLIKELY
+  throw_for_encoding_error("MULE_INTERNAL", buffer, start, 1);
 
   auto const byte2{get_byte(buffer, start + 1)};
   if (between_inc(byte1, 0x81, 0x8d) and byte2 >= 0xA0)
     return start + 2;
 
   if (start + 3 > buffer_len)
-    throw_for_encoding_error("MULE_INTERNAL", buffer, start, 2);
+    PQXX_UNLIKELY
+  throw_for_encoding_error("MULE_INTERNAL", buffer, start, 2);
 
   if (
     ((byte1 == 0x9A and between_inc(byte2, 0xa0, 0xdf)) or
@@ -446,7 +475,8 @@ PQXX_PURE std::size_t glyph_scanner<encoding_group::MULE_INTERNAL>::call(
     return start + 3;
 
   if (start + 4 > buffer_len)
-    throw_for_encoding_error("MULE_INTERNAL", buffer, start, 3);
+    PQXX_UNLIKELY
+  throw_for_encoding_error("MULE_INTERNAL", buffer, start, 3);
 
   if (
     ((byte1 == 0x9C and between_inc(byte2, 0xf0, 0xf4)) or
@@ -455,6 +485,7 @@ PQXX_PURE std::size_t glyph_scanner<encoding_group::MULE_INTERNAL>::call(
     get_byte(buffer, start + 4) >= 0xa0)
     return start + 4;
 
+  PQXX_UNLIKELY
   throw_for_encoding_error("MULE_INTERNAL", buffer, start, 4);
 }
 
@@ -485,7 +516,8 @@ PQXX_PURE std::size_t glyph_scanner<encoding_group::UHC>::call(
     return start + 1;
 
   if (start + 2 > buffer_len)
-    throw_for_encoding_error("UHC", buffer, start, buffer_len - start);
+    PQXX_UNLIKELY
+  throw_for_encoding_error("UHC", buffer, start, buffer_len - start);
 
   auto const byte2{get_byte(buffer, start + 1)};
   if (between_inc(byte1, 0x80, 0xc6))
@@ -495,13 +527,15 @@ PQXX_PURE std::size_t glyph_scanner<encoding_group::UHC>::call(
       between_inc(byte2, 0x80, 0xfe))
       return start + 2;
 
+    PQXX_UNLIKELY
     throw_for_encoding_error("UHC", buffer, start, 2);
   }
 
   if (between_inc(byte1, 0xa1, 0xfe))
   {
     if (not between_inc(byte2, 0xa1, 0xfe))
-      throw_for_encoding_error("UHC", buffer, start, 2);
+      PQXX_UNLIKELY
+    throw_for_encoding_error("UHC", buffer, start, 2);
 
     return start + 2;
   }
@@ -522,19 +556,22 @@ PQXX_PURE std::size_t glyph_scanner<encoding_group::UTF8>::call(
     return start + 1;
 
   if (start + 2 > buffer_len)
-    throw_for_encoding_error("UTF8", buffer, start, buffer_len - start);
+    PQXX_UNLIKELY
+  throw_for_encoding_error("UTF8", buffer, start, buffer_len - start);
 
   auto const byte2{get_byte(buffer, start + 1)};
   if (between_inc(byte1, 0xc0, 0xdf))
   {
     if (not between_inc(byte2, 0x80, 0xbf))
-      throw_for_encoding_error("UTF8", buffer, start, 2);
+      PQXX_UNLIKELY
+    throw_for_encoding_error("UTF8", buffer, start, 2);
 
     return start + 2;
   }
 
   if (start + 3 > buffer_len)
-    throw_for_encoding_error("UTF8", buffer, start, buffer_len - start);
+    PQXX_UNLIKELY
+  throw_for_encoding_error("UTF8", buffer, start, buffer_len - start);
 
   auto const byte3{get_byte(buffer, start + 2)};
   if (between_inc(byte1, 0xe0, 0xef))
@@ -542,11 +579,13 @@ PQXX_PURE std::size_t glyph_scanner<encoding_group::UTF8>::call(
     if (between_inc(byte2, 0x80, 0xbf) and between_inc(byte3, 0x80, 0xbf))
       return start + 3;
 
+    PQXX_UNLIKELY
     throw_for_encoding_error("UTF8", buffer, start, 3);
   }
 
   if (start + 4 > buffer_len)
-    throw_for_encoding_error("UTF8", buffer, start, buffer_len - start);
+    PQXX_UNLIKELY
+  throw_for_encoding_error("UTF8", buffer, start, buffer_len - start);
 
   if (between_inc(byte1, 0xf0, 0xf7))
   {
@@ -555,9 +594,11 @@ PQXX_PURE std::size_t glyph_scanner<encoding_group::UTF8>::call(
       between_inc(get_byte(buffer, start + 3), 0x80, 0xbf))
       return start + 4;
 
+    PQXX_UNLIKELY
     throw_for_encoding_error("UTF8", buffer, start, 4);
   }
 
+  PQXX_UNLIKELY
   throw_for_encoding_error("UTF8", buffer, start, 1);
 }
 
@@ -623,8 +664,9 @@ encoding_group enc_group(std::string_view encoding_name)
 
   auto const found_encoding_group{encoding_map.find(encoding_name)};
   if (found_encoding_group == std::end(encoding_map))
-    throw std::invalid_argument{
-      internal::concat("Unrecognized encoding: '", encoding_name, "'.")};
+    PQXX_UNLIKELY
+  throw std::invalid_argument{
+    internal::concat("Unrecognized encoding: '", encoding_name, "'.")};
   return found_encoding_group->second;
 }
 
@@ -644,7 +686,7 @@ constexpr inline F *for_encoding(encoding_group enc)
 
   switch (enc)
   {
-    CASE_GROUP(MONOBYTE);
+    PQXX_LIKELY CASE_GROUP(MONOBYTE);
     CASE_GROUP(BIG5);
     CASE_GROUP(EUC_CN);
     CASE_GROUP(EUC_JP);
@@ -658,8 +700,9 @@ constexpr inline F *for_encoding(encoding_group enc)
     CASE_GROUP(SJIS);
     CASE_GROUP(SHIFT_JIS_2004);
     CASE_GROUP(UHC);
-    CASE_GROUP(UTF8);
+    PQXX_LIKELY CASE_GROUP(UTF8);
   }
+  PQXX_UNLIKELY
   throw pqxx::usage_error{
     internal::concat("Unsupported encoding group code ", enc, ".")};
 
@@ -678,7 +721,7 @@ template<encoding_group E> struct char_finder
   constexpr static PQXX_PURE std::size_t
   call(std::string_view haystack, char needle, std::size_t start)
   {
-    auto const buffer{haystack.data()};
+    auto const buffer{std::data(haystack)};
     auto const size{std::size(haystack)};
     for (auto here{start}; here + 1 <= size;
          here = glyph_scanner<E>::call(buffer, size, here))
@@ -696,13 +739,13 @@ template<encoding_group E> struct string_finder
   PQXX_PURE constexpr static PQXX_PURE std::size_t
   call(std::string_view haystack, std::string_view needle, std::size_t start)
   {
-    auto const buffer{haystack.data()};
+    auto const buffer{std::data(haystack)};
     auto const size{std::size(haystack)};
     auto const needle_size{std::size(needle)};
     for (auto here{start}; here + needle_size <= size;
          here = glyph_scanner<E>::call(buffer, size, here))
     {
-      if (std::memcmp(buffer + here, needle.data(), needle_size) == 0)
+      if (std::memcmp(buffer + here, std::data(needle), needle_size) == 0)
         return here;
     }
     return std::string::npos;
