@@ -234,6 +234,7 @@ bool pqxx::pipeline::obtain_result(bool expect_none)
   {
     if (have_pending() and not expect_none)
     {
+      PQXX_UNLIKELY
       set_error_at(m_issuedrange.first->first);
       m_issuedrange.second = m_issuedrange.first;
     }
@@ -245,6 +246,7 @@ bool pqxx::pipeline::obtain_result(bool expect_none)
 
   if (not have_pending())
   {
+    PQXX_UNLIKELY
     set_error_at(std::begin(m_queries)->first);
     throw std::logic_error{
       "Got more results from pipeline than there were queries."};
@@ -252,7 +254,8 @@ bool pqxx::pipeline::obtain_result(bool expect_none)
 
   // Must be the result for the oldest pending query.
   if (not std::empty(m_issuedrange.first->second.res))
-    internal_error("Multiple results for one query.");
+    PQXX_UNLIKELY
+  internal_error("Multiple results for one query.");
 
   m_issuedrange.first->second.res = res;
   ++m_issuedrange.first;
@@ -272,8 +275,8 @@ void pqxx::pipeline::obtain_dummy()
   m_dummy_pending = false;
 
   if (r == nullptr)
-    internal_error(
-      "Pipeline got no result from backend when it expected one.");
+    PQXX_UNLIKELY
+  internal_error("Pipeline got no result from backend when it expected one.");
 
   result R{pqxx::internal::gate::result_creation::create(r, text, m_encoding)};
 
@@ -287,11 +290,14 @@ void pqxx::pipeline::obtain_dummy()
   {}
   if (OK)
   {
+    PQXX_LIKELY
     if (std::size(R) > 1)
-      internal_error("Unexpected result for dummy query in pipeline.");
+      PQXX_UNLIKELY
+    internal_error("Unexpected result for dummy query in pipeline.");
 
     if (R.at(0).at(0).as<std::string>() != theDummyValue)
-      internal_error("Dummy query in pipeline returned unexpected value.");
+      PQXX_UNLIKELY
+    internal_error("Dummy query in pipeline returned unexpected value.");
     return;
   }
 
