@@ -42,27 +42,29 @@ enum tx_stat
 };
 
 
+constexpr auto committed{"committed"_zv}, aborted{"aborted"_zv},
+  in_progress{"in progress"_zv};
+
+
 /// Parse a nonempty transaction status string.
 constexpr tx_stat parse_status(std::string_view text) noexcept
 {
-  constexpr auto committed{"committed"_zv}, aborted{"aborted"_zv},
-    in_progress{"in progress"_zv};
-
   switch (text[0])
   {
   case 'a':
     if (text == aborted)
-      return tx_aborted;
+      PQXX_LIKELY return tx_aborted;
     break;
   case 'c':
     if (text == committed)
-      return tx_committed;
+      PQXX_LIKELY return tx_committed;
     break;
   case 'i':
     if (text == in_progress)
-      return tx_in_progress;
+      PQXX_LIKELY return tx_in_progress;
     break;
   }
+  PQXX_UNLIKELY
   return tx_unknown;
 }
 
