@@ -123,18 +123,19 @@ wrap_to_chars(char *begin, char *end, T const &value)
 {
   auto res{std::to_chars(begin, end - 1, value)};
   if (res.ec != std::errc())
-    switch (res.ec)
-    {
-    case std::errc::value_too_large:
-      throw pqxx::conversion_overrun{
-        "Could not convert " + pqxx::type_name<T> +
-        " to string: "
-        "buffer too small (" +
-        pqxx::to_string(end - begin) + " bytes)."};
-    default:
-      throw pqxx::conversion_error{
-        "Could not convert " + pqxx::type_name<T> + " to string."};
-    }
+    PQXX_UNLIKELY
+  switch (res.ec)
+  {
+  case std::errc::value_too_large:
+    throw pqxx::conversion_overrun{
+      "Could not convert " + pqxx::type_name<T> +
+      " to string: "
+      "buffer too small (" +
+      pqxx::to_string(end - begin) + " bytes)."};
+  default:
+    throw pqxx::conversion_error{
+      "Could not convert " + pqxx::type_name<T> + " to string."};
+  }
   // No need to check for overrun here: we never even told to_chars about that
   // last byte in the buffer, so it didn't get used up.
   *res.ptr++ = '\0';
@@ -232,7 +233,7 @@ std::string demangle_type_name(char const raw[])
   // When __cxa_demangle fails, it's guaranteed to return null.
   char *demangled{abi::__cxa_demangle(raw, nullptr, nullptr, &status)};
 #else
-  constexpr char *demangled{nullptr};
+  static constexpr char *demangled{nullptr};
 #endif
   std::string const name{(demangled == nullptr) ? raw : demangled};
   std::free(demangled);
@@ -265,19 +266,20 @@ template<typename TYPE>
 [[maybe_unused]] inline TYPE from_string_arithmetic(std::string_view in)
 {
   char const *here;
-  auto const end{in.data() + std::size(in)};
+  auto const end{std::data(in) + std::size(in)};
 
   // Skip whitespace.  This is not the proper way to do it, but I see no way
   // that any of the supported encodings could ever produce a valid character
   // whose byte sequence would confuse this code.
-  for (here = in.data(); here < end and (*here == ' ' or *here == '\t');
+  for (here = std::data(in); here < end and (*here == ' ' or *here == '\t');
        ++here)
     ;
 
   TYPE out;
   auto const res{std::from_chars(here, end, out)};
   if (res.ec == std::errc() and res.ptr == end)
-    return out;
+    PQXX_LIKELY
+  return out;
 
   std::string msg;
   if (res.ec == std::errc())
@@ -322,15 +324,17 @@ template<typename T>
 {
   using limits = std::numeric_limits<T>;
 
-  constexpr T ten{10};
-  constexpr T high_threshold(std::numeric_limits<T>::max() / ten);
+  inline constexpr T ten{10};
+  inline constexpr T high_threshold(std::numeric_limits<T>::max() / ten);
   if (n > high_threshold)
-    report_overflow();
+    PQXX_UNLIKELY
+  report_overflow();
   if constexpr (limits::is_signed)
   {
-    constexpr T low_threshold(std::numeric_limits<T>::min() / ten);
+    inline constexpr T low_threshold(std::numeric_limits<T>::min() / ten);
     if (low_threshold > n)
-      report_overflow();
+      PQXX_UNLIKELY
+    report_overflow();
   }
   return T(n * ten);
 }
@@ -342,7 +346,8 @@ template<typename T>
 {
   T const high_threshold{static_cast<T>(std::numeric_limits<T>::max() - d)};
   if (n > high_threshold)
-    report_overflow();
+    PQXX_UNLIKELY
+  report_overflow();
   return static_cast<T>(n + d);
 }
 
@@ -353,7 +358,8 @@ template<typename T>
 {
   T const low_threshold{static_cast<T>(std::numeric_limits<T>::min() + d)};
   if (n < low_threshold)
-    report_overflow();
+    PQXX_UNLIKELY
+  report_overflow();
   return static_cast<T>(n - d);
 }
 
@@ -388,7 +394,7 @@ template<typename T>
     throw pqxx::conversion_error{
       "Attempt to convert empty string to " + pqxx::type_name<T> + "."};
 
-  char const *const data{text.data()};
+  char const *const data{std::data(text)};
   std::size_t i{0};
 
   // Skip whitespace.  This is not the proper way to do it, but I see no way
@@ -535,6 +541,7 @@ template<typename T> inline T from_string_awful_float(std::string_view text)
     }
     else
     {
+      PQXX_LIKELY
       if constexpr (have_thread_local)
       {
         thread_local dumb_stringstream<T> S;
@@ -640,11 +647,11 @@ template<typename T> std::string to_string_float(T value)
 {
 #if defined(PQXX_HAVE_CHARCONV_FLOAT)
   {
-    constexpr auto space{float_traits<T>::size_buffer(value)};
+    static constexpr auto space{float_traits<T>::size_buffer(value)};
     std::string buf;
     buf.resize(space);
     std::string_view const view{
-      float_traits<T>::to_buf(buf.data(), buf.data() + space, value)};
+      float_traits<T>::to_buf(std::data(buf), std::data(buf) + space, value)};
     buf.resize(std::end(view) - std::begin(view));
     return buf;
   }
