@@ -59,10 +59,10 @@ pqxx::stream_from::stream_from(
         m_glyph_scanner{get_scanner(tx)}
 {
   if (std::empty(columns))
-    tx.exec0(internal::concat("COPY "sv, table, " TO STDOUT"sv));
-  else
-    tx.exec0(
-      internal::concat("COPY "sv, table, "("sv, columns, ") TO STDOUT"sv));
+    PQXX_UNLIKELY
+  tx.exec0(internal::concat("COPY "sv, table, " TO STDOUT"sv));
+  else PQXX_LIKELY tx.exec0(
+    internal::concat("COPY "sv, table, "("sv, columns, ") TO STDOUT"sv));
   register_me();
 }
 
@@ -133,6 +133,7 @@ void pqxx::stream_from::close()
 {
   if (not m_finished)
   {
+    PQXX_UNLIKELY
     m_finished = true;
     unregister_me();
   }
@@ -171,7 +172,8 @@ void pqxx::stream_from::complete()
 void pqxx::stream_from::parse_line()
 {
   if (m_finished)
-    return;
+    PQXX_UNLIKELY
+  return;
   auto const next_seq{m_glyph_scanner};
 
   m_fields.clear();
@@ -189,8 +191,8 @@ void pqxx::stream_from::parse_line()
   // Make room for unescaping the line.  It's a pessimistic size.
   // Unusually, we're storing terminating zeroes *inside* the string.
   // This is the only place where we modify m_row.  MAKE SURE THE BUFFER DOES
-  // NOT GET RESIZED while we're working, because we're working with pointers
-  // into its buffer.
+  // NOT GET RESIZED while we're working, because we're working with views into
+  // its buffer.
   m_row.resize(line_size + 1);
 
   char const *line_begin{line.get()};
@@ -240,6 +242,7 @@ void pqxx::stream_from::parse_line()
         field_begin = write;
         break;
 
+        PQXX_UNLIKELY
       case '\\': {
         // Escape sequence.
         if (read >= line_end)
@@ -257,9 +260,11 @@ void pqxx::stream_from::parse_line()
           break;
 
         case 'b': // Backspace.
+          PQXX_UNLIKELY
           *write++ = '\b';
           break;
         case 'f': // Form feed
+          PQXX_UNLIKELY
           *write++ = '\f';
           break;
         case 'n': // Line feed.
@@ -276,6 +281,7 @@ void pqxx::stream_from::parse_line()
           break;
 
         default:
+          PQXX_LIKELY
           // Regular character ("self-escaped").
           *write++ = c;
           break;
@@ -283,6 +289,7 @@ void pqxx::stream_from::parse_line()
       }
       break;
 
+        PQXX_LIKELY
       default: *write++ = c; break;
       }
     }
