@@ -98,12 +98,13 @@ void pqxx::stream_to::complete()
 }
 
 
-void pqxx::stream_to::escape_field_to_buffer(std::string_view buf)
+void pqxx::stream_to::escape_field_to_buffer(std::string_view data)
 {
-  for (auto c : buf)
+  for (auto c : data)
   {
     switch (c)
     {
+      PQXX_UNLIKELY
     case '\b': m_buffer += "\\b"; break;  // Backspace
     case '\f': m_buffer += "\\f"; break;  // Vertical tab
     case '\n': m_buffer += "\\n"; break;  // Form feed
