@@ -58,8 +58,9 @@ pqxx::transaction_base::~transaction_base()
   try
   {
     if (not std::empty(m_pending_error))
-      process_notice(
-        internal::concat("UNPROCESSED ERROR: ", m_pending_error, "\n"));
+      PQXX_UNLIKELY
+    process_notice(
+      internal::concat("UNPROCESSED ERROR: ", m_pending_error, "\n"));
 
     if (m_registered)
     {
@@ -209,17 +210,9 @@ void pqxx::transaction_base::abort()
 }
 
 
-std::string pqxx::transaction_base::esc_raw(zview bin) const
-{
-  auto const p{reinterpret_cast<std::byte const *>(bin.c_str())};
-  return conn().esc_raw(std::basic_string_view<std::byte>{p, std::size(bin)});
-}
-
-
 std::string pqxx::transaction_base::quote_raw(zview bin) const
 {
-  auto const p{reinterpret_cast<std::byte const *>(bin.c_str())};
-  return conn().quote(std::basic_string_view<std::byte>{p, std::size(bin)});
+  return conn().quote(binary_cast(bin));
 }
 
 
@@ -291,11 +284,9 @@ void pqxx::transaction_base::check_rowcount_prepared(
   result::size_type actual_rows)
 {
   if (actual_rows != expected_rows)
-  {
     throw unexpected_rows{internal::concat(
       "Expected ", expected_rows, " row(s) of data from prepared statement '",
       statement, "', got ", actual_rows, ".")};
-  }
 }
 
 
@@ -303,11 +294,9 @@ void pqxx::transaction_base::check_rowcount_params(
   std::size_t expected_rows, std::size_t actual_rows)
 {
   if (actual_rows != expected_rows)
-  {
     throw unexpected_rows{internal::concat(
       "Expected ", expected_rows,
       " row(s) of data from parameterised query, got ", actual_rows, ".")};
-  }
 }
 
 
@@ -366,9 +355,10 @@ void pqxx::transaction_base::close() noexcept
       return;
 
     if (m_focus != nullptr)
-      m_conn.process_notice(internal::concat(
-        "Closing ", description(), "  with ", m_focus->description(),
-        " still open.\n"));
+      PQXX_UNLIKELY
+    m_conn.process_notice(internal::concat(
+      "Closing ", description(), "  with ", m_focus->description(),
+      " still open.\n"));
 
     try
     {
@@ -462,6 +452,7 @@ void pqxx::transaction_base::register_pending_error(zview err) noexcept
     {
       try
       {
+        PQXX_UNLIKELY
         process_notice("UNABLE TO PROCESS ERROR\n");
         process_notice(e.what());
         process_notice("ERROR WAS:");
@@ -486,6 +477,7 @@ void pqxx::transaction_base::register_pending_error(std::string &&err) noexcept
     {
       try
       {
+        PQXX_UNLIKELY
         process_notice("UNABLE TO PROCESS ERROR\n");
         process_notice(e.what());
         process_notice("ERROR WAS:");
