@@ -100,6 +100,7 @@ void pqxx::internal::check_unique_unregister(
 {
   if (new_guest != old_guest)
   {
+    PQXX_UNLIKELY
     if (new_guest == nullptr)
       throw usage_error{concat(
         "Expected to close ", describe_object(old_class, old_name),
@@ -117,12 +118,14 @@ void pqxx::internal::check_unique_unregister(
 
 namespace
 {
+constexpr char hex_digits[] = {'0', '1', '2', '3', '4', '5', '6', '7',
+                               '8', '9', 'a', 'b', 'c', 'd', 'e', 'f'};
+
+
 /// Translate a number (must be between 0 and 16 exclusive) to a hex digit.
 constexpr char hex_digit(int c) noexcept
 {
-  constexpr char hex[] = {'0', '1', '2', '3', '4', '5', '6', '7',
-                          '8', '9', 'a', 'b', 'c', 'd', 'e', 'f'};
-  return hex[c];
+  return hex_digits[c];
 }
 
 
@@ -130,19 +133,17 @@ constexpr char hex_digit(int c) noexcept
 constexpr int nibble(int c) noexcept
 {
   if (c >= '0' and c <= '9')
-    return c - '0';
-  else if (c >= 'a' and c <= 'f')
-    return 10 + (c - 'a');
-  else if (c >= 'A' and c <= 'F')
-    return 10 + (c - 'A');
-  else
-    return -1;
+    PQXX_LIKELY
+  return c - '0';
+  else if (c >= 'a' and c <= 'f') return 10 + (c - 'a');
+  else if (c >= 'A' and c <= 'F') return 10 + (c - 'A');
+  else return -1;
 }
 } // namespace
 
 
 void pqxx::internal::esc_bin(
-  std::string_view binary_data, char buffer[]) noexcept
+  std::basic_string_view<std::byte> binary_data, char buffer[]) noexcept
 {
   auto here{buffer};
   *here++ = '\\';
@@ -159,7 +160,8 @@ void pqxx::internal::esc_bin(
 }
 
 
-std::string pqxx::internal::esc_bin(std::string_view binary_data)
+std::string
+pqxx::internal::esc_bin(std::basic_string_view<std::byte> binary_data)
 {
   auto const bytes{size_esc_bin(std::size(binary_data))};
   std::string buf;
@@ -199,12 +201,13 @@ void pqxx::internal::unesc_bin(
 }
 
 
-std::string pqxx::internal::unesc_bin(std::string_view escaped_data)
+std::basic_string<std::byte>
+pqxx::internal::unesc_bin(std::string_view escaped_data)
 {
   auto const bytes{size_unesc_bin(std::size(escaped_data))};
-  std::string buf;
+  std::basic_string<std::byte> buf;
   buf.resize(bytes);
-  unesc_bin(escaped_data, reinterpret_cast<std::byte *>(buf.data()));
+  unesc_bin(escaped_data, buf.data());
   return buf;
 }
 
