@@ -215,18 +215,18 @@ void test_blob_read_span()
 
   std::vector<std::byte> vec_buf;
   vec_buf.resize(2);
-  output = b.read(vec_buf);
+  auto output2{b.read(vec_buf)};
   PQXX_CHECK_EQUAL(
-    std::size(output), 2u, "Got unexpected buf size from blob::read().");
+    std::size(output2), 2u, "Got unexpected buf size from blob::read().");
   PQXX_CHECK_EQUAL(
-    output[0], std::byte{'x'}, "Unexpected byte from blob::read().");
+    output2[0], std::byte{'x'}, "Unexpected byte from blob::read().");
   PQXX_CHECK_EQUAL(
-    output[1], std::byte{'y'}, "Unexpected byte from blob::read().");
+    output2[1], std::byte{'y'}, "Unexpected byte from blob::read().");
 
   vec_buf.resize(100);
-  output = b.read(vec_buf);
-  PQXX_CHECK_EQUAL(std::size(output), 1u, "Weird things happened at EOF.");
-  PQXX_CHECK_EQUAL(output[0], std::byte{'z'}, "Bad data at EOF.");
+  output2 = b.read(vec_buf);
+  PQXX_CHECK_EQUAL(std::size(output2), 1u, "Weird things happened at EOF.");
+  PQXX_CHECK_EQUAL(output2[0], std::byte{'z'}, "Bad data at EOF.");
 #endif // PQXX_HAVE_SPAN
 }
 
@@ -236,12 +236,17 @@ void test_blob_reads_vector()
   char const content[]{"abcd"};
   pqxx::connection conn;
   pqxx::work tx{conn};
-  auto id{pqxx::blob::from_buf(tx, std::basic_string_view<std::byte>{reinterpret_cast<std::byte const *>(content), std::size(content)})};
+  auto id{pqxx::blob::from_buf(
+    tx, std::basic_string_view<std::byte>{
+          reinterpret_cast<std::byte const *>(content), std::size(content)})};
   std::vector<std::byte> buf;
   buf.resize(10);
   auto out{pqxx::blob::open_r(tx, id).read(buf)};
-  PQXX_CHECK_EQUAL(std::size(out), std::size(content), "Got wrong length back when reading as vector.");
-  PQXX_CHECK_EQUAL(out[0], std::byte{'a'}, "Got bad data when reading as vector.");
+  PQXX_CHECK_EQUAL(
+    std::size(out), std::size(content),
+    "Got wrong length back when reading as vector.");
+  PQXX_CHECK_EQUAL(
+    out[0], std::byte{'a'}, "Got bad data when reading as vector.");
 }
 
 
@@ -279,6 +284,31 @@ void test_blob_write_appends_at_insertion_point()
 }
 
 
+void test_blob_writes_span()
+{
+#if defined(PQXX_HAVE_SPAN)
+  pqxx::connection conn;
+  pqxx::work tx{conn};
+  constexpr char content[]{"gfbltk"};
+  std::basic_string<std::byte> data{
+    reinterpret_cast<std::byte const *>(content), std::size(content)};
+
+  auto id{pqxx::blob::create(tx)};
+  auto b{pqxx::blob::open_rw(tx, id)};
+  b.write(std::span<std::byte>{data.data() + 1, 3u});
+  b.seek_abs(0);
+
+  std::vector<std::byte> buf;
+  buf.resize(4);
+  auto out{b.read(std::span<std::byte>{buf.data(), 4u})};
+  PQXX_CHECK_EQUAL(
+    std::size(out), 3u, "Did not get expected number of bytes back.");
+  PQXX_CHECK_EQUAL(out[0], std::byte{'f'}, "Data did not come back right.");
+  PQXX_CHECK_EQUAL(out[2], std::byte{'l'}, "Data started right, ended wrong!");
+#endif // PQXX_HAVE_SPAN
+}
+
+
 void test_blob_resize_shortens_to_desired_length()
 {
   std::basic_string<std::byte> const data{
@@ -417,6 +447,24 @@ void write_file(char const path[], std::basic_string_view<std::byte> data)
   }
   std::fclose(f);
 }
+
+
+/// Temporary file.
+class TempFile
+{
+public:
+  /// Create (and later clean up) a file at path containing data.
+  TempFile(char const path[], std::basic_string_view<std::byte> data) :
+          m_path(path)
+  {
+    write_file(path, data);
+  }
+
+  ~TempFile() { std::remove(m_path.c_str()); }
+
+private:
+  std::string m_path;
+};
 } // namespace
 
 
@@ -429,18 +477,12 @@ void test_blob_from_file_creates_blob_from_file_contents()
   pqxx::work tx{conn};
   std::basic_string<std::byte> buf;
 
-  try
+  pqxx::oid id;
   {
-    write_file(temp_file, data);
-    auto id{pqxx::blob::from_file(tx, temp_file)};
-    std::remove(temp_file);
-    pqxx::blob::to_buf(tx, id, buf, 10);
-  }
-  catch (std::exception const &)
-  {
-    std::remove(temp_file);
-    throw;
+    TempFile f{temp_file, data};
+    id = pqxx::blob::from_file(tx, temp_file);
   }
+  pqxx::blob::to_buf(tx, id, buf, 10);
   PQXX_CHECK_EQUAL(buf, data, "Wrong data from blob::from_file().");
 }
 
@@ -451,25 +493,18 @@ void test_blob_from_file_with_oid_writes_blob()
   char const temp_file[] = "blob-test-from_file-oid.tmp";
   std::basic_string<std::byte> buf;
 
-  try
+  pqxx::connection conn;
+  pqxx::work tx{conn};
+
+  // Guarantee (more or less) that id is not in use.
+  auto id{pqxx::blob::create(tx)};
+  pqxx::blob::remove(tx, id);
+
   {
-    pqxx::connection conn;
-    pqxx::work tx{conn};
-
-    // Guarantee (more or less) that id is not in use.
-    auto id{pqxx::blob::create(tx)};
-    pqxx::blob::remove(tx, id);
-
-    write_file(temp_file, data);
+    TempFile f{temp_file, data};
     pqxx::blob::from_file(tx, temp_file, id);
-    std::remove(temp_file);
-    pqxx::blob::to_buf(tx, id, buf, 10);
-  }
-  catch (std::exception const &)
-  {
-    std::remove(temp_file);
-    throw;
   }
+  pqxx::blob::to_buf(tx, id, buf, 10);
   PQXX_CHECK_EQUAL(buf, data, "Wrong data from blob::from_file().");
 }
 
@@ -539,6 +574,30 @@ void test_blob_close_leaves_blob_unusable()
 }
 
 
+void test_blob_accepts_std_filesystem_path()
+{
+#if __has_include(<filesystem>)
+  // A bug in gcc 8's ~std::filesystem::path() causes a run-time crash.
+#  if !defined(__GNUC__) || (__GNUC__ > 8)
+
+  char const temp_file[] = "blob-test-filesystem-path.tmp";
+  std::basic_string<std::byte> const data{std::byte{'4'}, std::byte{'2'}};
+
+  pqxx::connection conn;
+  pqxx::work tx{conn};
+  std::basic_string<std::byte> buf;
+
+  TempFile f{temp_file, data};
+  std::filesystem::path const path{temp_file};
+  auto id{pqxx::blob::from_file(tx, path)};
+  pqxx::blob::to_buf(tx, id, buf, 10);
+  PQXX_CHECK_EQUAL(buf, data, "Wrong data from blob::from_file().");
+
+#  endif
+#endif
+}
+
+
 PQXX_REGISTER_TEST(test_blob_is_useless_by_default);
 PQXX_REGISTER_TEST(test_blob_create_makes_empty_blob);
 PQXX_REGISTER_TEST(test_blob_create_with_oid_requires_oid_be_free);
@@ -552,6 +611,7 @@ PQXX_REGISTER_TEST(test_blob_read_reads_data);
 PQXX_REGISTER_TEST(test_blob_reads_vector);
 PQXX_REGISTER_TEST(test_blob_read_span);
 PQXX_REGISTER_TEST(test_blob_write_appends_at_insertion_point);
+PQXX_REGISTER_TEST(test_blob_writes_span);
 PQXX_REGISTER_TEST(test_blob_resize_shortens_to_desired_length);
 PQXX_REGISTER_TEST(test_blob_resize_extends_to_desired_length);
 PQXX_REGISTER_TEST(test_blob_tell_tracks_position);
@@ -563,4 +623,5 @@ PQXX_REGISTER_TEST(test_blob_from_file_with_oid_writes_blob);
 PQXX_REGISTER_TEST(test_blob_append_to_buf_appends);
 PQXX_REGISTER_TEST(test_blob_to_file_writes_file);
 PQXX_REGISTER_TEST(test_blob_close_leaves_blob_unusable);
+PQXX_REGISTER_TEST(test_blob_accepts_std_filesystem_path);
 } // namespace
