@@ -397,8 +397,12 @@ void test_stream_to_factory_with_dynamic_columns()
   tx.exec0("CREATE TEMP TABLE pqxx_stream_to(a integer, b varchar)");
 
   std::vector<std::string_view> columns{"a", "b"};
+#if defined(PQXX_HAVE_CONCEPTS)
+  auto stream{pqxx::stream_to::table(tx, {"pqxx_stream_to"}, columns)};
+#else
   auto stream{pqxx::stream_to::raw_table(
     tx, conn.quote_table({"pqxx_stream_to"}), conn.quote_columns(columns))};
+#endif
   stream.write_values(4, "four");
   stream.complete();
 
